/**
 * @file main.c
 * @author Jake G
 * @date 15 June 2024
 * @brief File contains the main function.
 *
 * This file contains the main logic loop and exists to setup the values
 * specific to the micro-controller. All other functions and configuration are
 * extracted into separate source files and headers for configuration.
 */

#include <stdio.h>
#define F_CPU 3333333UL

// This can prevent issues with utils/delay.h library with the gcc toolchain
#define __DELAY_BACKWARD_COMPATIBLE__

#include "LedController.h"
#include "RegEdit.h"
#include "SuperLoop.h"
#include "config.h"
#include "timer.h"
#include <avr/cpufunc.h> /* Required header file */
#include <avr/io.h>
#include <util/delay.h>

#define RELAYPIN (1 << 7)
#define RELAYREADINGPIN                                                        \
  (1 << 6) // It would be better to use PA7 so USART worked

// Set the function pointer for the delay func
void (*Delay_MicroSeconds)(double us) = _delay_us;

void Activate_Relay(void) { PORTA.OUT |= RELAYPIN; }

void Deactivate_Relay(void) { PORTA.OUT &= ~RELAYPIN; }

void WaitForRelayConnect(void) {
  while (!(PORTA.IN & RELAYREADINGPIN)) {
    ;
  }
}

void WaitForRelayDisconnect(void) {
  while (PORTA.IN & RELAYREADINGPIN) {
    ;
  }
}

/*
int main(int argc, char **argv) {
  // We use pin 10 which is the UDPI/RESET pin.
  // This means we have to change it's mode.

  PORTA.DIR |= RELAYPIN;
  PORTA.DIR &= ~RELAYREADINGPIN;

  uint16_t make_time = 0;
  uint16_t break_time = 0;
  uint8_t temp = 0;


  while (true) {
    Activate_Relay();

    Timer_Start();
    WaitForRelayConnect();
    Timer_Disable();

    make_time = Timer_GetOverflowCount();

    // Output the Make time via LEDS
    temp = (uint8_t)(make_time & 0xFF);

    Deactivate_Relay();

    Timer_Start();
    WaitForRelayDisconnect();
    Timer_Disable();

    break_time = Timer_GetOverflowCount();

    // Output the Break time via LEDS
    temp = (uint8_t)(break_time & 0xFF);
  }
}
*/

int main(int argc, char **argv)
{
    //Setup for Infinite Loop
    SuperLoop_SetIterations(0);
    SuperLoop_Run();
}
